import sys
from datetime import datetime

from git import Repo

from app.logger import Logger
from app.timer import Timer


class Manager:
    def __init__(
        self,
        logger: Logger,
        timer: Timer = Timer,
        path_repository=None,
        origin="origin",
    ):
        self.PREFIX_CLI = "pair/"
        self.origin = origin
        self.repository = Repo(path_repository, search_parent_directories=True)
        self.path_repository = path_repository
        self.logger = logger
        self.timer = timer
        self.DEFAULT_COMMIT_MESSAGE = "pair - [skip-cli]"

    def _get_remote(self):
        return self.repository.remote(self.origin)

    def _is_current_branch_pair(self):
        self.logger.debug(f"It's using the path: {self.path_repository}")
        self.logger.debug(f"It's working inside: {self.repository.working_tree_dir}")

        if self.PREFIX_CLI not in self.repository.active_branch.name:
            self.logger.error("You need to be inside a pair/ branch before execute it.")
            sys.exit(1)

    def _format_summary_date(self, date):
        return datetime.utcfromtimestamp(date).strftime("%Y-%m-%d %H:%M:%S")

    def run_timer(self, time_in_minutes: int):
        should_create_timer = isinstance(time_in_minutes, int) and time_in_minutes > 0
        if should_create_timer:
            time_to_seconds = time_in_minutes * 60
            self.timer.start_timer(time_to_seconds)
            self.logger.info("Created a timer 🕝. Relax, we will let you know!")

    def run_start(self, time_in_minutes=None):
        self.logger.debug("Fetching data")

        if self.repository.active_branch.is_remote():
            self._get_remote().fetch()

        first_time = False  # Responsible to determine a better message to show when is the first time

        # Only create a new branch if the current one hasn't the pair prefix
        if self.PREFIX_CLI not in self.repository.active_branch.name:
            first_time = True
            self.logger.debug("Creating branch")
            branch_name = self.PREFIX_CLI + self.repository.active_branch.name
            self.repository.git.checkout("-B", branch_name)
        else:
            self.logger.debug("Using same branch")
            branch_name = self.repository.active_branch.name

        if self.repository.active_branch.is_remote():
            self.logger.debug("Seems that already exists. Pulling it!")
            self._get_remote().pull(self.repository.active_branch.name)

        if len(self.repository.remotes):
            self.logger.debug("Pushing")
            self.repository.git.push(
                "--set-upstream", self._get_remote().name, branch_name
            )

        self.run_timer(time_in_minutes)

        # If we have a time set, start a timer!
        if first_time:
            self.logger.info(
                f"Done, branch '{branch_name}' created, happy pair programming 😄"
            )
            return

        self.logger.info("Sync done, happy pair programming 😄")

    def run_next(self):
        self._is_current_branch_pair()

        self.logger.debug("Adding all the files")
        self.repository.git.add(A=True)

        if len(self.repository.index.diff("HEAD")) >= 1:
            self.logger.debug("Commiting with pair default message, skipping the hooks")
            self.repository.git.commit("-m", self.DEFAULT_COMMIT_MESSAGE, "--no-verify")
        else:
            self.logger.debug("Nothing to commit.")

        if self.repository.active_branch.is_remote():
            self.logger.debug("Pushing")
            self.repository.git.push(
                "--set-upstream",
                self._get_remote().name,
                self.repository.active_branch.name,
            )

    def run_done(self):
        self._is_current_branch_pair()

        # To keep the origin updated
        self.run_next()

        pair_branch = self.repository.active_branch.name
        branch_without_prefix = self.repository.active_branch.name.replace(
            self.PREFIX_CLI, ""
        )

        self.logger.debug("Coming back to the original branch")
        self.repository.git.checkout(branch_without_prefix)

        if self.repository.active_branch.is_remote():
            self.logger.debug("Fetching and pulling the data")
            self._get_remote().fetch()
            self._get_remote().pull()

        self.logger.debug("Merging with the pair branch")
        self.repository.git.merge(pair_branch, no_ff=True, no_commit=True)

        if self.origin in self.repository.remotes:
            for ref in self.repository.remotes[self.origin].refs:
                if pair_branch not in ref.name:
                    continue

                self.logger.debug("Deleting pair branch from the remote environment")

                self.repository.git.push(
                    self._get_remote().name, "--delete", pair_branch
                )

        if pair_branch in self.repository.heads:
            self.logger.debug("Deleting pair branch from the local environment")
            self.repository.heads[pair_branch].delete(
                self.repository, pair_branch, force=True
            )

        self.logger.info("🌟 Done, continue with the git commit command.")

    def run_summary(self):
        self._is_current_branch_pair()

        all_track_items = []
        last_commit = None
        first_commit = None
        list_of_commits = list(
            self.repository.iter_commits(
                self.repository.active_branch.name, max_count=200
            )
        )

        for item in list_of_commits:
            # Get last commit
            if last_commit is None or last_commit.committed_date < item.committed_date:
                last_commit = item

            # Get first commit
            if (
                first_commit is None
                or item.committed_date < first_commit.committed_date
            ):
                first_commit = item

            commited_date_formatted = self._format_summary_date(item.committed_date)

            all_track_items.append(
                {
                    "committer": item.committer.email,
                    "start": commited_date_formatted,
                    "end": commited_date_formatted,
                }
            )

        # Last dev
        print("Last Dev:")
        last_date_dev = self._format_summary_date(last_commit.committed_date)

        print(
            "     {user: <25}".format(user=last_commit.committer.email),
            " |",
            last_date_dev,
        )

        # Started
        print("First Dev:")
        first_date_dev = self._format_summary_date(first_commit.committed_date)

        print(
            "     {user: <25}".format(user=first_commit.committer.email),
            " |",
            first_date_dev,
        )

        # Frequence
        print("Frequence:")

        # Group by user
        track_items_per_committer = {}
        for track in all_track_items:
            if track["committer"] in track_items_per_committer:
                track_items_per_committer[track["committer"]].append(track)
            else:
                track_items_per_committer[track["committer"]] = [track]

        for user in track_items_per_committer:
            total = len(track_items_per_committer[user])
            fill_total = [""] * (total)
            bar = "▇".join(fill_total)
            print("     {user: <25}".format(user=user), " |", bar, total)



import os

import click
from loguru import logger as LoguruLogger

from app.logger import Logger
from app.manager import Manager
from app.timer import Timer


@click.group()
# TODO: Move these options for each command, it's ugly make devpair -v start 3 instead of devpair start 3 -v
@click.option("-v", default=False, help="Debug all the steps", is_flag=True)
@click.option("-o", default="origin", help="Set the origin for the command")
@click.pass_context
def cli(ctx, v, o):
    ctx.ensure_object(dict)

    logger = Logger(logger=LoguruLogger)
    logger.set_verbose(v)

    timer = Timer(logger=logger)

    manager = Manager(path_repository=os.getcwd(), logger=logger, origin=o, timer=timer)

    # It's shared with other commands
    ctx.obj["MANAGER"] = manager


@cli.command()
@click.argument("time", required=False, type=int)
@click.pass_context
def start(ctx, time):
    """
    Start a new session inside the current branch.

    Try to use 'devpair start 10'. It will start the pairing but
    with a 10 minutes timer for you! In the end of the 10 minutes a
    voice will say 'Next dev'
    """

    ctx.obj["MANAGER"].run_start(time)


@cli.command()
@click.pass_context
def next(ctx):
    """
    Save everything and send to the next person.
    """

    ctx.obj["MANAGER"].run_next()


@cli.command()
@click.pass_context
def done(ctx):
    """
    Finish and put all the work in the original branch.
    """
    ctx.obj["MANAGER"].run_done()


@cli.command()
@click.argument("time", required=False, type=int)
@click.pass_context
def timer(ctx, time):
    """
    Create a simple timer, it's good when you forget to put
    a timer value inside the devpair start <TIMER>
    """

    ctx.obj["MANAGER"].run_timer(time)


@cli.command()
@click.pass_context
def summary(ctx):
    """
    Show a simple resume of commit counts and last developer.
    """

    ctx.obj["MANAGER"].run_summary()



import sys
from loguru import logger


class Logger:
    def __init__(self, logger: logger) -> None:
        self.logger = logger
        self._setup_default_format()

    def _setup_default_format(self) -> None:
        """Define the initial format for loguru"""
        fmt = "<green>{time:YYYY-MM-DD HH:mm:ss}</green> | <level>{level: <8}</level> - <level>{message}</level>"
        self.logger.remove()
        self.logger.add(sys.stderr, format=fmt)

    def set_verbose(self, active: bool) -> None:
        """If true print all the statements inside the code"""
        if not active:
            self.logger.remove()
            self.logger.add(sys.stderr, level="INFO")

    def error(self, message) -> None:
        self.logger.error(message)

    def debug(self, message) -> None:
        self.logger.debug(message)

    def info(self, message) -> None:
        self.logger.info(message)

    def warn(self, message) -> None:
        self.logger.warn(message)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['app']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.29,<4.0.0',
 'click>=8.0.4,<9.0.0',
 'loguru>=0.6.0,<0.7.0',
 'python-dateutil>=2.8.2,<3.0.0']

entry_points = \
{'console_scripts': ['devpair = devpair:exec_cli']}

setup_kwargs = {
    'name': 'devpair',
    'version': '1.1.9',
    'description': 'Pair script that manage a pair programming session using git.',
    'long_description': '<div align="center">\n    <img src="./logo.png" width="60px">\n</div>\n\n## Dev Pair\n\n[![Python Test](https://github.com/raphaelkieling/pair/actions/workflows/push.yml/badge.svg)](https://github.com/raphaelkieling/pair/actions/workflows/push.yml)\n![PyPI Version](https://img.shields.io/pypi/v/devpair)\n![Python Version](https://img.shields.io/pypi/pyversions/devpair)\n\nIt\'s a tool to facilitate the remote pair programming session. Instead of make a lot of `git add, git commit, git push, git pull`, you can make it more quickly only running `devpair start <timer optional>` to start coding and `devpair next` to send the code to another person.\n\nVery useful for teams that like to make pair sessions often. If you never was a driver or a navigator feel free to read [here](https://martinfowler.com/articles/on-pair-programming.html) to have a context.\n\n## Install\n\n```\npip install devpair\n```\n\n## Commands\n\n- `devpair start <time in minutes optional>` Start a new session inside the current branch.\n- `devpair next` Save everything and send to the next person.\n- `devpair done` Finish and put all the work in the original branch.\n- `devpair summary` Show a simple resume of commit counts and last developer.\n- `devpair timer <time in minutes>` Create a simple timer for any purposal.\n- `devpair --help` I like to have a help some times.\n\n> We recommend to create some alias if you want, like: `dvs` to run `devpair start`, `dvn` to run `devpair next` and `dvd` to run `devpair done`.\n\n## Example of use\n\n```bash\n# Dev A\nmain $ devpair start 10 # Will start the new branch with a 10 minutes timer\npair/main $ echo "hello" > welcome.txt\npair/main $ devpair next\n\n# Dev B\nmain $ devpair start\npair/main $ cat welcome.txt # shows "hello"\npair/main $ echo " world" >> welcome.txt\npair/main $ devpair next\n\n# Dev A\npair/main $ devpair start\npair/main $ cat welcome.txt # shows "hello world"\npair/main $ echo "!" >> welcome.txt\npair/main $ devpair done\n\nmain $ git commit -m "feat: created hello world feature"\nmain $ git push\n\n# Dev B\npair/main $ devpair done # just to come back to the original branch\nmain\n```\n\n## How it works internally?\n\nYou will work inside a temporary pair branch that in the end all the commits will be squashed to be added to the feature branch.\n\n<details>\n    <summary>MORE INFO PLEASE!</summary>\n\nOk ok! Under the hood the `devpair start` will take your current branch and create a copy with the same name but with the prefix `pair`\n\nAfter make your code changes the `devpair next` will add, commit and push your code using an internal commit message. This step will be more easier to understand checking the [example step by step](#example-of-use)\n\nIn the end, we have the `devpair done` that will add, commit, push and delete the branch. Don\'t worry we will make a squash commit of everything that you did for the current branch.\n\n</details>\n\n[![](https://mermaid.ink/img/pako:eNqNkMEKwjAMhl9l5Dzx3rPgA3jtJbb_1uLajpgiMvbu1oOgDGE5fSTfn0AWcsWDDI1Rz8JzsLlr5UpKUbd8Fc4udBmPwwDWKtjlzxzluDv0wwHuVqpuTybIiH-bP6nEMX_rG5N6apOm-faD5d2zpAEJlkxDj4HrpJZsXpvKVcvlmR0ZlYqe6uxZcYo8CicyA093rC_K-3GZ?type=png)](https://mermaid.live/edit#pako:eNqNkMEKwjAMhl9l5Dzx3rPgA3jtJbb_1uLajpgiMvbu1oOgDGE5fSTfn0AWcsWDDI1Rz8JzsLlr5UpKUbd8Fc4udBmPwwDWKtjlzxzluDv0wwHuVqpuTybIiH-bP6nEMX_rG5N6apOm-faD5d2zpAEJlkxDj4HrpJZsXpvKVcvlmR0ZlYqe6uxZcYo8CicyA093rC_K-3GZ)\n\n### Recommendations\n\n- Before the pair programming\n  - Define the end of the session. How many time do you want pair?\n  - Define the break time.\n- Use a `timer`, maybe `devpair` timer or any other one, the important thing is to keep the pair rotation:\n  - `devpair start <time in minutes>`\n  - https://cuckoo.team/\n  - https://double-trouble.wielo.co/\n  - http://mobtimer.zoeetrope.com/\n  - ANY other mobile app, web tool, smartwatch app, pomodoro timer and so on.\n- The `driver` need to share the screen avoiding to use tools like `vscode live share`, even they are good it can create some hard moments that you want to show the browser or create a quickly diagram. The preference is that the `driver` ever need to share the screen.\n- Antipatterns: https://tuple.app/pair-programming-guide/antipatterns\n\n### Contributing\n\nFork, create a branch from `main` with the pattern `feat/my-feature` and make a pull request with your proposal.\n\n### Local env\n\nWe are using [poetry](https://python-poetry.org/) and [pyenv](https://github.com/pyenv/pyenv) to manage all the python versions and dependencies.\n\n```sh\n# Install all the dependencies\npoetry install\n# Install the pre-commit\npoetry run pre-commit install\n# Set the version of the `.python-version`\npyenv local\n# Run all the tests\nmake unit\n```\n\n### Publishing\n\n```sh\nsh ./scripts/tag-new-version.bash v0.0.0\n```\n',
    'author': 'raphael.kieling',
    'author_email': 'raphael.kieling@telus.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import os
import platform
from app.logger import Logger


class Timer:
    def __init__(self, logger: Logger) -> None:
        self.logger = logger

    def _execute_unix_timer(self, time: int) -> None:
        commands = [
            f"sleep {time}",
            "say Next dev",
        ]

        joined_command = " && ".join(commands)

        os.system(f"""sh -c '{joined_command}' &""")

    def _get_current_system(self) -> str:
        return platform.system()

    def start_timer(self, time: int):
        system = self._get_current_system()

        if system == "Darwin" or system == "Linux":
            self._execute_unix_timer(time)
            return

        # TODO: Add support for windows
        self.logger.warn("Currently we only have the timer for MacOS and Linux")
